
From: James Morris <jmorris@redhat.com>

This patch adds a file to the root selinuxfs directory which returns the
security policy version associated with the currently running kernel.  Its
purpose is to allow scripts to determine which version of policy to load.



 security/selinux/include/security.h |    1 
 security/selinux/selinuxfs.c        |   44 +++++++++++++++++++++++++++++++++++-
 security/selinux/ss/policydb.h      |    1 
 3 files changed, 44 insertions(+), 2 deletions(-)

diff -puN security/selinux/include/security.h~selinux-add-policyvers security/selinux/include/security.h
--- 25/security/selinux/include/security.h~selinux-add-policyvers	2003-10-12 17:57:39.000000000 -0700
+++ 25-akpm/security/selinux/include/security.h	2003-10-12 17:57:39.000000000 -0700
@@ -13,6 +13,7 @@
 #define SECCLASS_NULL			0x0000 /* no class */
 
 #define SELINUX_MAGIC 0xf97cff8c
+#define POLICYDB_VERSION 15
 
 #ifdef CONFIG_SECURITY_SELINUX_BOOTPARAM
 extern int selinux_enabled;
diff -puN security/selinux/selinuxfs.c~selinux-add-policyvers security/selinux/selinuxfs.c
--- 25/security/selinux/selinuxfs.c~selinux-add-policyvers	2003-10-12 17:57:39.000000000 -0700
+++ 25-akpm/security/selinux/selinuxfs.c	2003-10-12 17:57:39.000000000 -0700
@@ -37,7 +37,8 @@ enum sel_inos {
 	SEL_ACCESS,	/* compute access decision */
 	SEL_CREATE,	/* compute create labeling decision */
 	SEL_RELABEL,	/* compute relabeling decision */
-	SEL_USER	/* compute reachable user contexts */
+	SEL_USER,	/* compute reachable user contexts */
+	SEL_POLICYVERS	/* return policy version for this kernel */
 };
 
 static ssize_t sel_read_enforce(struct file *filp, char *buf,
@@ -125,6 +126,46 @@ static struct file_operations sel_enforc
 	.write		= sel_write_enforce,
 };
 
+static ssize_t sel_read_policyvers(struct file *filp, char *buf,
+                                   size_t count, loff_t *ppos)
+{
+	char *page;
+	ssize_t length;
+	ssize_t end;
+
+	if (count < 0 || count > PAGE_SIZE)
+		return -EINVAL;
+	if (!(page = (char*)__get_free_page(GFP_KERNEL)))
+		return -ENOMEM;
+	memset(page, 0, PAGE_SIZE);
+
+	length = snprintf(page, PAGE_SIZE, "%u", POLICYDB_VERSION);
+	if (length < 0) {
+		free_page((unsigned long)page);
+		return length;
+	}
+
+	if (*ppos >= length) {
+		free_page((unsigned long)page);
+		return 0;
+	}
+	if (count + *ppos > length)
+		count = length - *ppos;
+	end = count + *ppos;
+	if (copy_to_user(buf, (char *) page + *ppos, count)) {
+		count = -EFAULT;
+		goto out;
+	}
+	*ppos = end;
+out:
+	free_page((unsigned long)page);
+	return count;
+}
+
+static struct file_operations sel_policyvers_ops = {
+	.read		= sel_read_policyvers,
+};
+
 static ssize_t sel_write_load(struct file * file, const char * buf,
 			      size_t count, loff_t *ppos)
 
@@ -568,6 +609,7 @@ static int sel_fill_super(struct super_b
 		[SEL_CREATE] = {"create", &transaction_ops, S_IRUGO|S_IWUGO},
 		[SEL_RELABEL] = {"relabel", &transaction_ops, S_IRUGO|S_IWUGO},
 		[SEL_USER] = {"user", &transaction_ops, S_IRUGO|S_IWUGO},
+		[SEL_POLICYVERS] = {"policyvers", &sel_policyvers_ops, S_IRUGO},
 		/* last one */ {""}
 	};
 	return simple_fill_super(sb, SELINUX_MAGIC, selinux_files);
diff -puN security/selinux/ss/policydb.h~selinux-add-policyvers security/selinux/ss/policydb.h
--- 25/security/selinux/ss/policydb.h~selinux-add-policyvers	2003-10-12 17:57:39.000000000 -0700
+++ 25-akpm/security/selinux/ss/policydb.h	2003-10-12 17:57:39.000000000 -0700
@@ -225,7 +225,6 @@ extern int policydb_read(struct policydb
 
 #define PERM_SYMTAB_SIZE 32
 
-#define POLICYDB_VERSION 15
 #define POLICYDB_CONFIG_MLS    1
 
 #define OBJECT_R "object_r"

_
